//#define TESTOWANIE
#define PRZEKAZNIK

/*
 * Obsuga tablicy wietlnej
 * UART_ATtiny2313.c
 *
 * Created: 2014-12-22
 * Author: Jacek
 */ 


#define F_CPU 11059200UL  // 11,0592MHz

#include <avr/io.h>
#include <util/delay.h>
#include <avr/interrupt.h>


/*****************************************************************
  FUNKCJE ZWIZANE Z OBSUG USART0
 *****************************************************************/
//zmienna pomocnicza, do odczytu statusu itp.
uint8_t temp;

//obliczenie nastawy rejestru UBRR
#define USART_BAUDRATE 115200
#define BAUD_PRESCALE ((F_CPU/(USART_BAUDRATE*16UL))-1)

void USART_Init(void)
{
	//ustawienie prdkoci transmisji
	UBRRL = BAUD_PRESCALE;
	UBRRH = (BAUD_PRESCALE >> 8);
	//format ramki: 115200, 8 bitw, parzysto=n, bit stopu=1
	//zezwolenie na prac odbiornika/nadajnika, ew. przyjmowanie przerwa
	UCSRB = ((1<<TXEN)|(1<<RXEN) | (1<<RXCIE));
}

void USART_SendByte(uint8_t u8Data)
{
	// Wait until last byte has been transmitted
	while((UCSRA &(1<<UDRE)) == 0);
	// Transmit data
	UDR = u8Data;
}

void USART_SendString(char * T)
{
	while (*T != 0)
	{
		USART_SendByte(*T);
		T++;
	}
}

// not being used but here for completeness
// Wait until a byte has been received and return received data 
uint8_t USART_ReceiveByte()
{
	while((UCSRA &(1<<RXC)) == 0);
	return UDR;
}


/*****************************************************************
  FUNKCJE ZWIZANE Z OBSUG WYWIETLACZA LED
 *****************************************************************/
//                                ENQ   ESC   1 <- pocztek tekstu
//                                                  ESC   0 <- offset LSB
//                                                              ESC   0 <- offset MSB
const char message_PREAMBULA[] = {0x05, 0x1B, 0xFE, 0x1B, 0xFF, 0x1B, 0xFF, 0};
const char message_BRAK_MIEJSC[] = {'<', 'Y', '0', '1', '>', '<', 'T', '1', '>', '<', 'F', '0', '0', '>', 'B', 'R', 'A', 'K', ' ', 'M', 'I', 'E', 'J', 'S', 'C', '<', 'N', 'U', 'L', 'L', '>', 0};
const char message_KONIEC_TEKSTU[] = {0x05, 0x1B, 0xFD, 0xFE, 0x04, 0};
const char message_INTENSYWNOSC_AUTO[] = {0x05, 0x1B, 0xF4, 0xFF, 0xF6, 0x04, 0};

//bufor na komunikat w RAM
#define	indeks_dziesiatek	24
#define indeks_jednostek	indeks_dziesiatek+1
//char do_wyswietlenia[] = {'<', 'Y', '0', '1', '>', '<', 'T', '1', '>', '<', 'F', '0', '0', '>', 'W', 'O', 'L', 'N', 'E', ':', ' ', ' ', '<', 'N', 'U', 'L', 'L', '>', 0};

char do_wyswietlenia[] = {'<', 'C', 'L', 'R', '>', '<', 'S', '0', '0', '3', ' ', '0', '0', '1', ' ', '0', '0', ' ', 'W', 'O', 'L', 'N', 'E', ':', '-', '-', '>', '<', 'E', '0', '3', '>', 0};

//obliczenie sumy kontrolnej (suma bajtw, dopenienie jedynkowe po zsumowaniu)
uint8_t CRC(char * T)
{
	uint8_t suma_CRC = 0;
	
	while (*T != 0)
	{
		suma_CRC = suma_CRC + (uint8_t)(*T);
		T++;
	}
	suma_CRC = suma_CRC & 0xFF;
	suma_CRC = ~suma_CRC;
	return (suma_CRC);
}

//adresowanie wywietlacza, oczekiwanie na ACK (=0)
void LED_SendAddress()
{
#ifndef TESTOWANIE
	do
	{
		//adresowanie wywietlacza
#endif
		USART_SendByte(0x16);
		USART_SendByte(0x20);
		//oczekiwanie na odpowied (0 = status gotowy)
#ifndef TESTOWANIE
		temp = USART_ReceiveByte();
		temp = USART_ReceiveByte();
	} while (temp != 0);
#endif
}

//ustawienie automatycznej regulacji jasnoci wiecenia
void LED_SetIntensityAuto()
{
	LED_SendAddress();
	USART_SendString(message_INTENSYWNOSC_AUTO);
#ifndef TESTOWANIE
	temp = USART_ReceiveByte();
#endif
}

//wysanie komunikatu "BRAK MIEJSC"
void LED_BrakMiejsc()
{
	//adresowanie wywietlacza (adres stay)
	LED_SendAddress();
	//wysanie komunikatu
	USART_SendString(message_PREAMBULA);
	USART_SendString(message_BRAK_MIEJSC);
	//wysanie koca tekstu
	USART_SendByte(0x1B);
	USART_SendByte(0xFF);
	//obliczenie i wysanie CRC
	USART_SendByte(CRC(message_BRAK_MIEJSC));
	//wysanie znacznika EOT
	USART_SendByte(0x04);
	//koniec wprowadzania tekstu - wywietlenie
	USART_SendString(message_KONIEC_TEKSTU);
#ifndef TESTOWANIE
	temp = USART_ReceiveByte();
#endif
}

//wysanie informacji o liczbie wolnych miejsc
void LED_SaMiejsca(uint8_t liczba)
{
	char dziesiatki, jednostki, suma_crc;
	//formatowanie komunikatu
	if (liczba < 10)
	{
		dziesiatki = liczba + '0';
		jednostki = ' ';
	}
	else
	{
		dziesiatki = liczba / 10 + '0';
		jednostki = liczba % 10 + '0';
	}
	do_wyswietlenia[indeks_dziesiatek] = dziesiatki;
	do_wyswietlenia[indeks_jednostek] = jednostki;
	//adresowanie wywietlacza (adres stay)
	LED_SendAddress();
	//wysanie komunikatu
	USART_SendString(message_PREAMBULA);
	USART_SendString(do_wyswietlenia);
	//wysanie koca tekstu
	USART_SendByte(0x1B);
	USART_SendByte(0xFF);
	//obliczenie i wysanie CRC (wartoci z zakresu 0..1F musz by poprzedzone ESC
	suma_crc = CRC(do_wyswietlenia);
	if (suma_crc <= 0x1F) USART_SendByte(0x1B);
	USART_SendByte(suma_crc);
	//wysanie znacznika EOT
	USART_SendByte(0x04);
	//koniec wprowadzania tekstu - wywietlenie
	USART_SendString(message_KONIEC_TEKSTU);
#ifndef TESTOWANIE
	//tu dziesiatki uyto jako zmienna tymczasowa
	dziesiatki = USART_ReceiveByte();
#endif
}


/*****************************************************************
  OBSUGA I USTAWIENIE TRYBW PRACY PORTW I/O
 *****************************************************************/
//zmienna dla liczby odczytanej z wej
uint8_t	LICZBA_MIEJSC = 0, STEROWANIE;
//definicja wyjcia
#define WYJSCIE		6
#define DIODALED	2
#define LED_WYL		PORTD |= (1<<DIODALED)
#define LED_ZAL		PORTD &= ~(1<<DIODALED)
#define WYJSCIE_ZAL	PORTD |= (1<<WYJSCIE)
#define WYJSCIE_WYL	PORTD &= ~(1<<WYJSCIE)


//ustawienie trybw pracy portw
void ports_init()
{
	//port A - wartoci domylne
	//cay port B jako wejciowy
	DDRB = 0;
	PORTB = 0xFF;
	//port C - brak wyprowadze
	//port D - 0, 2..5 - wejcia
	//       - 1, 6 - wyjcia
	//       - 7 - brak wyprowadze
	DDRD = 0b01000110;
	PORTD = 0b01000100;
}


/*****************************************************************
  PROGRAM GWNY (OBSUGA SYGNAW POBUDZAJCYCH)
 *****************************************************************/
#define IMPULS_ZEGAROWY	0x08			//impuls zegarowy na PB3


int main(void)
{
	ports_init();						//inicjowanie portw I/O
	WYJSCIE_WYL;
	LED_WYL;
	
	USART_Init();						//inicjowanie USART
	LED_SetIntensityAuto();				//ustawienie intensywnoci wiecenia na "auto"
	LED_BrakMiejsc();					//wywietlenie liczby dostpnych miejsc
	LED_ZAL;							//zaczenie diody LED
	
/***************************
        PTLA GWNA
 ***************************/
	for (;;)
	{
		STEROWANIE = PINB;				//odczyt portu D - sygnay sterujce
		STEROWANIE = ~STEROWANIE;		//negowanie zmiennej, eby aktywne byy "1"

		//**** zwikszanie stanu licznika
		if (STEROWANIE & IMPULS_ZEGAROWY)
		{
#ifdef PRZEKAZNIK
			_delay_ms(50);
#endif
			LED_WYL;					//zgaszenie diody LED
			STEROWANIE = PINB;
			LICZBA_MIEJSC = ~STEROWANIE;	//w zmiennej STEROWANIE ju jest odczytany PINB
										//bity danych s ustalone po ustawieniu linii IMPULS_ZEGAROWY
			LICZBA_MIEJSC &= 0x07;		//maskowanie bitw od 3 do 7

			do
			{
				STEROWANIE = PINB;
				STEROWANIE = ~STEROWANIE;
			} while (STEROWANIE & IMPULS_ZEGAROWY);

			STEROWANIE = STEROWANIE << 3;	//dane przesyane poracjami po 3 bity, std przesuniecie o 3 miejsca w lewo
			STEROWANIE = STEROWANIE & 0b00111000;	//maskowanie bitw b0...b3, b6...b7
			LICZBA_MIEJSC = LICZBA_MIEJSC + STEROWANIE;	//obliczenie liczby wysanej przez sterownik PLC
			if (LICZBA_MIEJSC > 0) LED_SaMiejsca(LICZBA_MIEJSC);
				else LED_BrakMiejsc();
			_delay_ms(300);
			LED_ZAL;
		}
    }
}
